#ifndef BOARD_H
#define BOARD_H
// Copyright (c) 2005 David Drysdale.  See license.txt for license info

#include <PalmOS.h>
#include "section.h"

#define BUILDDATESIZE 16
typedef struct Board {
  // ** First, fields that are fixed for the particular game type **
  // Length of this structure, including the cell array at the end.  This should be 
  // sizeof(BOARD) + width*height*sizeof(UInt8)
  Int16 length;
  // Null-terminated string with the build date in it
  // This should always be the second field in this 
  // structure, and is used to ensure that the database
  // information is compatible with the current version
  // of the application.
  Char buildDate[BUILDDATESIZE];
  // Width of the board
  Int16 width;
  // Height of the board
  Int16 height;
  // Total number of mines on the board
  Int16 totalMines;

  // ** Preferences and best times **
  Int16 smallEasyTime;
  Int16 smallMediumTime;
  Int16 smallHardTime;
  Int16 mediumEasyTime;
  Int16 mediumMediumTime;
  Int16 mediumHardTime;
  Int16 largeEasyTime;
  Int16 largeMediumTime;
  Int16 largeHardTime;
  UInt16 preferences;
#define DOQUESTIONMARKS   0x0001

  // ** Fields that change as a single game progresses **
  // Game status
#define GAMESTARTEDMASK   0x0001
#define CELLEXPOSEDMASK   0x0002
#define GAMELOSTMASK      0x0004
#define GAMEWONMASK       0x0008
#define GAMEOVERMASK      (GAMEWONMASK|GAMELOSTMASK)
  UInt16 gameStatus;
  // Amount of time taken so far, in ticks
  UInt32 timeTaken;
  // Number of markers left to place
  Int16 minesLeftToMark;
  // Number of unflagged cells still to reveal
  Int16 unflaggedCellsToReveal;
  // Cell statuses
  // The following array actually contains (width*height) entries
  UInt8 cells[1];
} BOARD;

#define GAMESTARTED(B)  ((B)->gameStatus & GAMESTARTEDMASK)
#define GAMEOVER(B)     ((B)->gameStatus & GAMEOVERMASK)
#define GAMEWON(B)      ((B)->gameStatus & GAMEWONMASK)
#define GAMELOST(B)     ((B)->gameStatus & GAMELOSTMASK)

#define CELLVALUE(B,C,R)  (B)->cells[((B)->width*(R))+(C)]

// First 5 bits are only changed at board initialization time
#define NBRSMASK      0x0F
#define MINEMASK      0x10
// Next bit changes from 0 to 1 at most once during the game
#define REVEALEDMASK  0x20
// Last two bits can change at any time, provided that the REVEALED
// bit is not set
#define QUESTIONMASK  0x40
#define FLAGMASK      0x80

#define NBRS(C)       ((C) & NBRSMASK)
#define MINE(C)       ((C) & MINEMASK)
#define REVEALED(C)   ((C) & REVEALEDMASK)
#define QUESTION(C)   ((C) & QUESTIONMASK)
#define FLAGGED(C)    ((C) & FLAGMASK)

BOARD *CreateBoard(Int16 width, Int16 height, Int16 numMines) SECT_BOARD;
void CopyPreferences(BOARD *dst, BOARD *src) SECT_BOARD;
void CopyBoard(BOARD *dst, BOARD *src) SECT_BOARD;
void ResetTimes(BOARD *board) SECT_BOARD;
void InitBoard(BOARD *board) SECT_BOARD;
void MakeCellSafe(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
void FlagCell(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
void QuestionCell(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
void UnflagCell(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
void Toggle2Cell(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
void Toggle3Cell(BOARD *board, Int16 col, Int16 row) SECT_BOARD;
Int16 ExposeCell(BOARD *board, Int16 col, Int16 row, Int16 *pExposeCount) SECT_BOARD;
Int16 DeduceCell(BOARD *board, Int16 col, Int16 row, Int16 *pExposeCount) SECT_BOARD;
void ShowAllMines(BOARD *board) SECT_BOARD;

#endif
